
' ---------------------------------------------------------------------------
' Spiel "Meteor-Mess" auf dem I2C-LCD einer Open-Macro - Version 1.1
' In OCBASIC 1.08a von Dietmar Harlos ab 29. Mrz 2012 bis 2. April 2012
' ---------------------------------------------------------------------------

' "Meteor-Mess" ist der Name eines Spiels, das ich um 1990 aus der Zeit-
' schrift "Pocket Computer" der Firma Fischel aus Berlin abgetippt habe. Es
' lief auf einem CASIO FX-850P. Ich habe es auf die Open-Macro umgesetzt, um
' die I2C-LCD-Routinen zu testen. Gesteuert wird es mit den Tasten an PORT[4]
' und PORT[5]. Es wird ein LCD mit 4*20 Zeichen verwendet, das mit dem Port-
' expander PCF8574 auf die gleiche Weise verbunden ist wie bei dem I2C-LCD
' von Conrad Electronic (Bestellnr. 198330). Eine kleine Platine mit 16-pol.
' Wannenstecker fr das LCD und Sockel fr den PCF ist schnell auf einem
' Stck Lochrasterplatine aufgebaut.

' Bei dieser Version 1.1 werden die Tasten etwas ergonomischer abgefragt.
' Auerdem habe ich etwas ber die Entprellung der Tasten geschrieben.

' ---------------------------------------------------------------------------

' Deklarationen und Definitionen

INCLUDE "omac.def"

DEFINE keydown PORT[4]               'Taste fr "nach unten"
DEFINE keyup PORT[5]                 'Taste fr "nach oben"
DEFINE beeper PORT[6]                'Piezo-Tner

DEFINE bufstart 50                   'Im USER-RAM liegt der 4*20=80 Zeichen
                                     ' groe Buffer
DIM punkte WORD
DIM a,b,x,y BYTE
DIM ^q BYTE
DIM ^z BYTE
DIM position BYTE

DIM bitarray BYTE
DIM keyupold   BIT[1] OF bitarray    'Alte Zustnde der Tasten
DIM keydownold BIT[2] OF bitarray

' ---------------------------------------------------------------------------

' Hauptprogramm

init

REPEAT
  output
  handlekeys
  ADD punkte,1
UNTIL nextstep

lcd_goto(1,2)
SYS lcd_string : INLINE " Punkte: ",0
printpunkte
lcd_put ASC(" ")

End2Host=ON
END

' ---------------------------------------------------------------------------

' einige Initialisierungen durchfhren

PROCEDURE init
  keyupold=keyup
  keydownold=keydown
  position=0
  punkte=0
  initbuffer
  lcd_init
  RETURN
END PROCEDURE

' ---------------------------------------------------------------------------

' Buffer initialisieren

PROCEDURE initbuffer
  z=bufstart
  FOR a=0 TO 79
    @z=ASC(" ")
    ADD z,1
  NEXT a
  RETURN
END PROCEDURE

' ---------------------------------------------------------------------------

' Buffer auf dem LCD ausgeben

PROCEDURE output
  q=bufstart
  FOR b=0 TO 3
    lcd_goto(b,0)
    FOR a=0 TO 19
      lcd_put @q
      ADD q,1
    NEXT a
  NEXT b
  RETURN
END PROCEDURE

' ---------------------------------------------------------------------------

' Buffer nach links scrollen

PROCEDURE scroll
  q=bufstart+1
  z=bufstart
  FOR a=0 TO 78
    @z=@q
    ADD q,1
    ADD z,1
  NEXT a
  RETURN
END PROCEDURE

' ---------------------------------------------------------------------------

' Den nchsten Schritt ausfhren
' bei einer Kollision wird der Wert 1 zurckgeliefert, sonst 0

FUNCTION nextstep
  scroll                             'Buffer nach links scrollen

  z=bufstart+19                      'rechten Rand mit SPACE berschreiben
  FOR b=0 TO 3
    @z=ASC(" ")
    ADD z,20
  NEXT b

  IF RAND>(24000-punkte*10) THEN     'Zahl ndern fr mehr oder weniger Meteore
    b=(RAND SHR 14) AND 3 '0..3
    z=bufstart+19+b*20
    @z=ASC("*")                      'zufallsbedingt Meteore ("*") setzen
  END IF

  q=bufstart+position*20

  IF @q=ASC("*") THEN                'Kollisionen feststellen
    @q=ASC("+")
    output
    BEEP 250, 25, beeper
    RETURN 1
  END IF

  @q=ASC(">")                        'Raumschiff im Buffer positionieren

  RETURN 0
END FUNCTION

' ---------------------------------------------------------------------------

' Tasten abfragen

' Stichwort "Entprellung": Die Tasten prellen, aber wenn die Tasten nicht zu
'  hufig abgefragt werden, wirkt das wie eine Entprellung. In diesem
'  Programm werden die Tasten ca. 8 mal pro Sekunde abgefragt. Also etwa
'  einmal alle 125 ms.

PROCEDURE handlekeys
  IF keyup<>keyupold THEN            'Zustand verndert?
    keyupold=keyup
    IF NOT(keyup) THEN               'gedrckt
      IF position THEN SUB position,1
    END IF
  END IF

  IF keydown<>keydownold THEN        'Zustand verndert?
    keydownold=keydown
    IF NOT(keydown) THEN             'gedrckt
      IF position<3 THEN ADD position,1
    END IF
  END IF

  RETURN
END PROCEDURE

' ---------------------------------------------------------------------------

' Anzahl der erreichten Punkte im LCD ausgeben

PROCEDURE printpunkte
  lcd_put (punkte/10000 + ASC("0"))
  lcd_put ((punkte/1000) MOD 10 + ASC("0"))
  lcd_put ((punkte/100) MOD 10 + ASC("0"))
  lcd_put ((punkte/10) MOD 10 + ASC("0"))
  lcd_put (punkte MOD 10 + ASC("0"))
  RETURN
END PROCEDURE

' ---------------------------------------------------------------------------

' Position im LCD setzen
' es sollte auch ein "lcd_putcmd &h80+64*(y AND 1)+20*(y/2)+x" mglich sein
' die DD-RAM-Adresse der verschiedenen Zeilen ist u.U. LCD-abhngig

PROCEDURE lcd_goto(y,x)
  IF y=0 THEN
    lcd_putcmd &h80+x                'LCD-Kommando "Set DD-RAM address"
  ELSE IF y=1 THEN
    lcd_putcmd &hc0+x
  ELSE IF y=2 THEN
    lcd_putcmd &h94+x
  ELSE IF y=3 THEN
    lcd_putcmd &hd4+x
  END IF
  RETURN
END PROCEDURE

' ---------------------------------------------------------------------------
' Die I2C-LCD-Routinen einbinden
' ---------------------------------------------------------------------------

INCLUDE "i2clcd.pro"

' ---------------------------------------------------------------------------
' Die I2C-Routinen fuer BYTEPORT[2] einbinden
' ---------------------------------------------------------------------------

INCLUDE "om_i2c_b.pro"

' ---------------------------------------------------------------------------
' Die Firmware-Routinen einbinden
' ---------------------------------------------------------------------------

INCLUDE "om_fw.pro"



