
' ----------------------------------------------------------------------------
' Kleine Pizza-Uhr in OCBASIC 1.07a auf der Open-Macro am 22. Juli 2010
' Benutzt das I2C-LCD von Conrad Electronic (Bestellnr. 198330) an BYTEPORT[2]
' ----------------------------------------------------------------------------

INCLUDE "omac.def"                   'Definitionen fr die Open-Macro

DIM tastestart PORT[3]               'Portbelegung
DIM tasteup    PORT[4]
DIM tastedown  PORT[5]
DIM beeper     PORT[6]

DIM bitarray BYTE
DIM tastestartalt BIT[1] OF bitarray 'Alte Zustnde der Tasten
DIM tasteupalt    BIT[2] OF bitarray
DIM tastedownalt  BIT[3] OF bitarray
DIM signalstart   BIT[4] OF bitarray 'Signale
DIM signalup      BIT[5] OF bitarray
DIM signaldown    BIT[6] OF bitarray
DIM signalsekunde BIT[7] OF bitarray
DIM signalreinit  BIT[8] OF bitarray

DIM zustand BYTE                     'Variablen
DIM temp BYTE
DIM sekundealt BYTE
DIM anzsekunden INT

DEFINE zuseinstellen 1               'Zustnde des Zustandsautomaten
DEFINE zuscountdown 2
DEFINE zusalarm 3

' ----------------------------------------------------------------------------

PRINT "Pizza-Uhr"

' Schleife des Hauptprogramms

WHILE TRUE

  init

  lcd_putcmd &h80                    'LCD-Kommando "Set DD-RAM address"
  SYS lcd_string : INLINE "Pizza-Uhr von D.H.",0   'Text auf dem LCD ausgeben

  anzsekunden=18*60                  'Defaultwert: 18 Minuten
  showrestzeit

  WHILE signalreinit=OFF
    tastenabfrage
    uhrabfrage
    zustandsautomat
  WEND

WEND

' ----------------------------------------------------------------------------

' Verschiedene Initialisierungen

PROCEDURE init
  tastestartalt=tastestart
  tasteupalt=tasteup
  tastedownalt=tastedown
  signalstart=OFF
  signalup=OFF
  signaldown=OFF
  signalreinit=OFF
  zustand=zuseinstellen
  lcd_init
  RETURN
END PROCEDURE


' Tasten abfragen und Signale signalstart, signalup und signaldown erzeugen

PROCEDURE tastenabfrage
  IF tastestart<>tastestartalt THEN  'Zustand verndert?
    tastestartalt=tastestart
    IF tastestart THEN               'losgelassen
      signalstart=ON
    END IF
  END IF

  IF tasteup<>tasteupalt THEN        'Zustand verndert?
    tasteupalt=tasteup
    IF tasteup THEN                  'losgelassen
      signalup=ON
    END IF
  END IF

  IF tastedown<>tastedownalt THEN    'Zustand verndert?
    tastedownalt=tastedown
    IF tastedown THEN                'losgelassen
      signaldown=ON
    END IF
  END IF

  RETURN
END PROCEDURE


' Uhr abfragen und Signal signalsekunde erzeugen

PROCEDURE uhrabfrage
  temp=SECOND
  IF temp<>sekundealt THEN           'Uhr wurde weitergezhlt?
    sekundealt=temp
    signalsekunde=ON
  END IF

  RETURN
END PROCEDURE


' Zustandsautomat

PROCEDURE zustandsautomat

  IF zustand=zuseinstellen THEN      'Zustand "Einstellen"

    IF signalstart THEN
      signalstart=OFF
      signalsekunde=OFF
      zustand=zuscountdown
    END IF

    IF signalup THEN
      signalup=OFF
      IF anzsekunden<99*60 THEN
        ADD anzsekunden,60
        showrestzeit
      ELSE
        BEEP 30,2,beeper
      END IF
    END IF

    IF signaldown THEN
      signaldown=OFF
      IF anzsekunden>60 THEN
        SUB anzsekunden,60
        showrestzeit
      ELSE
        BEEP 30,2,beeper
      END IF
    END IF

  ELSEIF zustand=zuscountdown THEN   'Zustand "Count Down"

    IF signalsekunde THEN
      signalsekunde=OFF
      SUB anzsekunden,1
      showrestzeit
    END IF

    IF anzsekunden=0 THEN
      signalstart=OFF
      zustand=zusalarm
    END IF

  ELSEIF zustand=zusalarm THEN       'Zustand "Alarm"

    IF signalsekunde THEN
      signalsekunde=OFF
      BEEP 30,5,beeper
    END IF

    IF signalstart THEN
      signalreinit=ON
    END IF

  END IF

  RETURN
END PROCEDURE


' Die "anzsekunden" in Minuten im Format 00:00 anzeigen

PROCEDURE showrestzeit
  lcd_putcmd &hc0       'Kommando "Set DD-RAM address" zum LCD senden
  lcd_put (((anzsekunden/60)/10)+ASC("0"))
  lcd_put (((anzsekunden/60) MOD 10)+ASC("0"))
  lcd_put ASC(":")
  lcd_put (((anzsekunden MOD 60)/10)+ASC("0"))
  lcd_put (((anzsekunden MOD 60) MOD 10)+ASC("0"))
  RETURN
END PROCEDURE





' ---------------------------------------------------------------------------
' Die I2C-LCD-Routinen einbinden
' ---------------------------------------------------------------------------

INCLUDE "i2clcd.pro"

' ---------------------------------------------------------------------------
' Die I2C-Routinen fuer BYTEPORT[2] einbinden
' ---------------------------------------------------------------------------

INCLUDE "om_i2c_b.pro"

' ---------------------------------------------------------------------------
' Die Firmware-Routinen einbinden
' ---------------------------------------------------------------------------

INCLUDE "om_fw.pro"




